extern crate bcrypt;

use crate::models;
use bcrypt::verify;
use diesel::prelude::*;
use jsonwebtoken::{decode, encode, DecodingKey, EncodingKey, Header, Validation};
use serde::{Deserialize, Serialize};
use std::error::Error;
use std::fmt;

#[derive(Debug, Serialize, Deserialize)]
struct Claims {
    issuer: String,
    user: String,
    is_admin: bool,
    exp: usize,
}

#[derive(Debug)]
struct PasswordMatchError;

impl fmt::Display for PasswordMatchError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "Passwords do not match")
    }
}

impl Error for PasswordMatchError {
    fn source(&self) -> Option<&(dyn Error + 'static)> {
        None
    }
}

pub fn new_token(
    uname: &str,
    password: &str,
    conn: &MysqlConnection,
) -> Result<String, Box<dyn Error + Send>> {
    use crate::schema::users::dsl::*;
    let user = users
        .filter(username.eq(uname))
        .first::<models::User>(conn)
        .unwrap();

    if !verify(password, &user.password_hash).unwrap() {
        return Err(Box::new(PasswordMatchError));
    }

    let secret = std::env::var("SECRET").expect("SECRET");
    let key = secret.as_bytes();

    let my_claims = Claims {
        issuer: "example.com".to_owned(),
        user: user.username.to_owned(),
        is_admin: user.is_admin.to_owned(),
        exp: 10000000000,
    };

    match encode(
        &Header::default(),
        &my_claims,
        &EncodingKey::from_secret(key),
    ) {
        Ok(t) => Ok(t),
        Err(err) => Err(Box::new(err)),
    }
}

pub fn validate_token(token: &str) -> Result<bool, jsonwebtoken::errors::Error> {
    let secret = std::env::var("SECRET").expect("SECRET");
    let key = secret.as_bytes();

    let validation = Validation::default();
    match decode::<Claims>(&token, &DecodingKey::from_secret(key), &validation) {
        Ok(c) => return Ok(true),
        Err(e) => return Err(e),
    };
}
